var GraphTreeSearch = function () {};

/*
 * Breadth First Search - http://en.wikipedia.org/wiki/Breadth-first_search
 * @graph, the graph we need to manipulate
 * @searchFunction, this function will check if the current result match with what we want to search. Must return a boolean;
 * @childsContainerName, the property who contains the child's array
 * @exitOnFirst, specify if the function must exit on first result found
 */
GraphTreeSearch.prototype.bfs = function (graph, searchFunction, childsContainerName, exitOnFirst) {
	var res = [];
	if(graph[childsContainerName]) {
		var queue = [];
		queue.push(graph[childsContainerName]);
		while(queue.length) {
			var el = queue.shift(),
				i = 0,
				l = el.length;
			for(; i<l; i++) {
				if(searchFunction(el[i])) {
					res.push(el[i]);
					if(exitOnFirst) {
						queue = [];
						break;
					}
				}
				if(el[i][childsContainerName]) {
					queue.push(el[i][childsContainerName]);
				}
			}
		}
	}
	return res;
}

/*
 * Depth First Search - http://en.wikipedia.org/wiki/Depth-first_search
 * @graph, the graph we need to manipulate
 * @searchFunction, this function will check if the current result match with what we want to search. Must return a boolean;
 * @childsContainerName, the property who contains the child's array
 * @exitOnFirst, specify if the function must exit on first result found
 */
GraphTreeSearch.prototype.dfs = function (graph, searchFunction, childsContainerName, exitOnFirst) {
	var res = [];
	if(graph[childsContainerName]) {
		var el = graph[childsContainerName],
			i = 0,
			l = el.length;
		for(;i < l; i++) {
			if(searchFunction(el[i])) {
				res.push(el[i]);
			}
			if(exitOnFirst && res.length) {
				break;
			} else {
				var tmp = this.dfs(el[i], searchFunction, childsContainerName, exitOnFirst),
					j = tmp.length,
					z = 0;
				for(;z < j; z++) {
					res.push(tmp[z]);
				}
				if(res.length && exitOnFirst) {
					break;
				}
			}
		}
	}
	return res;
};

module.exports = GraphTreeSearch;