var assert = require('assert'),
	GraphTreeSearch = require('../lib/GraphTreeSearch'),
	data = require('../utils/GraphData');

describe("[Lib] Graph & Tree search util", function () {
	var gts;
	beforeEach(function () {
		gts = new GraphTreeSearch();
	});
	describe("Searching using Breadth First Search algorithm", function () {
		it("should be able to find a specific element", function () {
			var result = gts.bfs(data, function (data) {
				if(data.name === "third") {
					return true;
				}
				return false;
			}, "friends", false);
			assert.equal(result.length, 8);
			result = gts.bfs(data, function (data) {
				if(data.name === "first") {
					return true;
				}
				return false;
			}, "friends", false);
			assert.equal(result.length, 2);
		});
		it("should be able to exit on first result", function () {
			var result = gts.bfs(data, function (data) {
				if(data.name === "third") {
					return true;
				}
				return false;
			}, "friends", true);
			assert.equal(result.length, 1);
		});
		it("should be able to return an empty array if no results were found", function () {
			var result = gts.bfs(data, function (data) {
				if(data.name === "fakeName") {
					return true;
				}
				return false;
			}, "friends", false);
			assert.equal(result.length, 0);
		});
	});
	describe("Searching using Depth First Search algorithm", function () {
		it("should be able to find a specific element", function () {
			var result = gts.dfs(data, function (data) {
				if(data.name === "third") {
					return true;
				}
				return false;
			}, "friends", false);
			assert.equal(result.length, 8);
			result = gts.dfs(data, function (data) {
				if(data.name === "first") {
					return true;
				}
				return false;
			}, "friends", false);
			assert.equal(result.length, 2);
		});
		it("should be able to exit on first result", function () {
			var result = gts.dfs(data, function (data) {
				if(data.name === "third") {
					return true;
				}
				return false;
			}, "friends", true);
			assert.equal(result.length, 1);
		});
		it("should be able to return an empty array if no results were found", function () {
			var result = gts.dfs(data, function (data) {
				if(data.name === "fakeName") {
					return true;
				}
				return false;
			}, "friends", false);
			assert.equal(result.length, 0);
		});
	});
});