var assert = require('assert'),
	LinkedList = require('../lib/LinkedList');

describe('[Lib] Linked List', function () {
	it('should be able instantiate a list object of size 0', function () {
		var list = new LinkedList();
		assert.equal(0, list.size());
		assert.equal("object", typeof list);
	});
	it('should be able to add elements to a list and update the actual list size correctly', function () {
		var list = new LinkedList();
		list.add("first");
		list.add("second");
		list.add("third");
		list.add("fourth");
		list.add("fifth");
		assert.equal(5, list.size());
	});
	it('should be able to add and retrieve elements based on their index', function () {
		var list = new LinkedList();
		list.add("first");
		list.add("second");
		list.add("third");
		list.add("fourth");
		assert.equal("first", list.get(0));
		assert.equal("second", list.get(1));
		assert.equal("third", list.get(2));
		assert.equal("fourth", list.get(3));
	});
	it('should return null values if get(index) is out of range', function () {
		var list = new LinkedList();
		list.add("first");
		list.add("second");
		list.add("third");
		list.add("fourth");
		assert.equal(null, list.get(5));
		assert.equal(null, list.get(-1));
	});
	it('should return the value of the element removed and update the list size', function () {
		var list = new LinkedList();
		list.add("first");
		list.add("second");
		list.add("third");
		list.add("fourth");
		assert.equal("second", list.remove(1));
		assert.equal(3, list.size());
		assert.equal("third", list.remove(1));
		assert.equal(2, list.size());
		assert.equal("first", list.remove(0));
		assert.equal(1, list.size());
		assert.equal("fourth", list.remove(0));
		assert.equal(0, list.size());
		assert.equal(null, list.remove(0));
	});
	it('should be able to remove starting from head and tail elements', function () {
		var list = new LinkedList();
		list.add("first");
		list.add("second");
		list.add("third");
		list.add("fourth");
		list.add("fifth");
		list.add("sixth");
		assert.equal("fifth", list.remove(4));
		assert.equal("fourth", list.remove(3));
		assert.equal("sixth", list.remove(3));
		assert.equal("third", list.remove(2));
	});
});